'use strict';

// src/params-patcher.ts
function visitNode(value, keyPath, onIntegerOverflow) {
  if (Array.isArray(value)) {
    return value.map(
      (element, ii) => visitNode(element, [...keyPath, ii], onIntegerOverflow)
    );
  } else if (typeof value === "object" && value !== null) {
    const out = {};
    for (const propName in value) {
      if (Object.prototype.hasOwnProperty.call(value, propName)) {
        out[propName] = visitNode(value[propName], [...keyPath, propName], onIntegerOverflow);
      }
    }
    return out;
  } else if (typeof value === "bigint") {
    if (onIntegerOverflow && (value > Number.MAX_SAFE_INTEGER || value < -Number.MAX_SAFE_INTEGER)) {
      onIntegerOverflow(keyPath, value);
    }
    return Number(value);
  } else {
    return value;
  }
}
function patchParamsForSolanaLabsRpc(params, onIntegerOverflow) {
  return visitNode(params, [], onIntegerOverflow);
}

// src/response-patcher-types.ts
var KEYPATH_WILDCARD = {};

// src/response-patcher-allowed-numeric-values.ts
var ALLOWED_NUMERIC_KEYPATHS = {
  getInflationReward: [[KEYPATH_WILDCARD, "commission"]]
};

// src/response-patcher.ts
function getNextAllowedKeypaths(keyPaths, property) {
  return keyPaths.filter((keyPath) => keyPath[0] === KEYPATH_WILDCARD && typeof property === "number" || keyPath[0] === property).map((keyPath) => keyPath.slice(1));
}
function visitNode2(value, allowedKeypaths) {
  if (Array.isArray(value)) {
    return value.map((element, ii) => {
      const nextAllowedKeypaths = getNextAllowedKeypaths(allowedKeypaths, ii);
      return visitNode2(element, nextAllowedKeypaths);
    });
  } else if (typeof value === "object" && value !== null) {
    const out = {};
    for (const [propName, innerValue] of Object.entries(value)) {
      const nextAllowedKeypaths = getNextAllowedKeypaths(allowedKeypaths, propName);
      out[propName] = visitNode2(innerValue, nextAllowedKeypaths);
    }
    return out;
  } else if (typeof value === "number" && // The presence of an allowed keypath on the route to this value implies it's allowlisted;
  // Upcast the value to `bigint` unless an allowed keypath is present.
  allowedKeypaths.length === 0) {
    return BigInt(value);
  } else {
    return value;
  }
}
function patchResponseForSolanaLabsRpc(rawResponse, methodName) {
  const allowedKeypaths = methodName ? ALLOWED_NUMERIC_KEYPATHS[methodName] : void 0;
  return visitNode2(rawResponse, allowedKeypaths ?? []);
}

// src/index.ts
function createSolanaRpcApi(config) {
  return new Proxy({}, {
    defineProperty() {
      return false;
    },
    deleteProperty() {
      return false;
    },
    get(...args) {
      const [_, p] = args;
      const methodName = p.toString();
      return function(...rawParams) {
        const handleIntegerOverflow = config?.onIntegerOverflow;
        const params = patchParamsForSolanaLabsRpc(
          rawParams,
          handleIntegerOverflow ? (keyPath, value) => handleIntegerOverflow(methodName, keyPath, value) : void 0
        );
        return {
          methodName,
          params,
          responseProcessor: (rawResponse) => patchResponseForSolanaLabsRpc(rawResponse, methodName)
        };
      };
    }
  });
}

exports.createSolanaRpcApi = createSolanaRpcApi;
//# sourceMappingURL=out.js.map
//# sourceMappingURL=index.node.cjs.map